#!/bin/sh

#
# ビルドしたりなんだり
#

##################################################
# プログラムの個別設定
##################################################
# jad 設定
MIDLET_NAME="MScripter demo"
FILE_NAME="msc_demo"
MIDLET_VERSION=0.2.0
MIDLET_VENDOR="epii"
MIDLET_ICON=icon.png
MIDLET_DESCRIPTION="MScripter demo"
MIDLET_DATA_SIZE=
MIDLET_INFO_URL="http://epii.homelinux.net/"

BASE_PACKAGE=net.homelinux.epii.mscripter
MAIN_CLASS_NAME=MScripterMIDlet

SOURCE_ENCODING=utf8
CONFIGURATION=CLDC-1.1
PROFILE=MIDP-2.0

##################################################
# WTK の設定
##################################################
WTK_DIR=~/WTK2.5.2
BOOTCLASSPATH=${WTK_DIR}/lib/cldcapi11.jar:${WTK_DIR}/lib/midpapi20.jar

##################################################
# ディレクトリの設定
##################################################
CURRENT_DIR=`pwd`
WORKING_DIR=work
RESOURCE_DIR=res
SOURCE_DIR=src
BINARY_DIR=${WORKING_DIR}/bin
RELEASE_DIR=release
JAVADOC_DIR=doc

CLASSES_DIR=${WORKING_DIR}/classes
TMPCLASS_DIR=${CLASSES_DIR}/tmp
PROCLASS_DIR=${CLASSES_DIR}/proguard
VERIFIEDCLASS_DIR=${CLASSES_DIR}/verified

##################################################
# ファイルパスの設定
##################################################
MANIFEST_PATH=${WORKING_DIR}/MANIFEST.MF
JAR_PATH=${BINARY_DIR}/${FILE_NAME}.jar
JAD_PATH=${BINARY_DIR}/${FILE_NAME}.jad
IN_JAR_PATH=${WORKING_DIR}/in.jar
OUT_JAR_PATH=${WORKING_DIR}/out.jar

##################################################
# コマンド設定
##################################################
RM="rm -rf"
MKDIR="mkdir -p"
CD=cd
JAVA=java
JAVAC=javac
JAVADOC=javadoc
JAR=jar
EMULATOR=emulator
PREVERIFY=preverify
PROGUARD=proguard

manifest(){
	echo " * マニュフェストファイルを作成しています..."
	${RM} ${MANIFEST_PATH}
	${MKDIR} ${WORKING_DIR}
	echo "MIDlet-1: ${MIDLET_NAME}, ${MIDLET_ICON}, ${BASE_PACKAGE}.${MAIN_CLASS_NAME}" >> ${MANIFEST_PATH}
	echo "MIDlet-Name: ${MIDLET_NAME}" >> ${MANIFEST_PATH}
	echo "MIDlet-Vendor: ${MIDLET_VENDOR}" >> ${MANIFEST_PATH}
	echo "MIDlet-Version: ${MIDLET_VERSION}" >> ${MANIFEST_PATH}
	echo "MicroEdition-Configuration: ${CONFIGURATION}" >> ${MANIFEST_PATH}
	echo "MicroEdition-Profile: ${PROFILE}" >> ${MANIFEST_PATH}
}

compile(){
	echo " * コンパイルしています..."
	${RM} ${TMPCLASS_DIR}
	${MKDIR} ${TMPCLASS_DIR}
	${JAVAC} -encoding ${SOURCE_ENCODING} \
	         -source 1.3 -target 1.3 \
	         -bootclasspath ${BOOTCLASSPATH} \
	         -d ${TMPCLASS_DIR} \
	         -classpath ${SOURCE_DIR} \
	         ${SOURCE_DIR}/`echo ${BASE_PACKAGE} | sed y/\./\\\//`/${MAIN_CLASS_NAME}.java
}

optimize(){
	echo " * 最適化しています..."
	${JAR} cfm ${IN_JAR_PATH} ${MANIFEST_PATH} -C ${TMPCLASS_DIR} .
	${PROGUARD} -injar ${IN_JAR_PATH} \
	            -outjar ${OUT_JAR_PATH} \
	            -libraryjar ${BOOTCLASSPATH} \
	            -allowaccessmodification \
	            -dontpreverify \
	            -overloadaggressively \
	            -optimizationpasses 2 \
	            -repackageclasses '' \
	            -keep "class * extends javax.microedition.midlet.MIDlet"
	${MKDIR} ${PROCLASS_DIR}
	${CD} ${PROCLASS_DIR}
	${JAR} xf ${CURRENT_DIR}/${OUT_JAR_PATH}
	${CD} ${CURRENT_DIR}
}

verify(){
	echo " * クラスファイルを検証しています..."
	${RM} ${VERIFIEDCLASS_DIR}
	${MKDIR} ${VERIFIEDCLASS_DIR}
	${PREVERIFY} -classpath ${BOOTCLASSPATH} \
	             -d ${VERIFIEDCLASS_DIR} \
	             ${VERIFYCLASS_DIR}
}

reduce(){
	echo " * シナリオファイルをリデュースしています..."
#	${JAVAC} -d ${WORKING_DIR} ScenarioReducer.java
	${JAVA} -classpath ${WORKING_DIR} ScenarioReducer 0.txt res/0.txt
}

makejar(){
	echo " * Jar ファイルを作成しています..."
	${RM} ${JAR_PATH}
	${MKDIR} ${BINARY_DIR}
	${JAR} cfm ${JAR_PATH} ${MANIFEST_PATH} \
	       -C ${VERIFIEDCLASS_DIR} . -C ${RESOURCE_DIR} .
	# Jar ファイルサイズ取得
	JAR_SIZE=`wc -c ${JAR_PATH} | awk '{print $1}'`
	if test ${JAR_SIZE} -gt `expr 1024 \* 300`
	then
	echo " **************************************************"
	echo " * Jar ファイルサイズオーバー: ${JAR_SIZE} bytes"
	echo " **************************************************"
	else
	echo " * Jar ファイルサイズは ${JAR_SIZE} bytes です"
	fi
}

makejad(){
	echo " * Jad ファイルを作成しています..."
	${RM} ${JAD_PATH}
	# 必須項目 (au)
	echo "MIDlet-Name: ${MIDLET_NAME}" >> ${JAD_PATH}
	echo "MIDlet-Version: ${MIDLET_VERSION}" >> ${JAD_PATH}
	echo "MIDlet-Vendor: ${MIDLET_VENDOR}" >> ${JAD_PATH}
	echo "MIDlet-Jar-URL: ${FILE_NAME}.jar" >> ${JAD_PATH}
	echo "MIDlet-Jar-Size: ${JAR_SIZE}">> ${JAD_PATH}
	echo "MIDlet-1: ${MIDLET_NAME}, ${MIDLET_ICON}, ${BASE_PACKAGE}.${MAIN_CLASS_NAME}" >> ${JAD_PATH}
	# 必須でない項目
	if test -n "${MIDLET_ICON}"; then echo "MIDlet-Icon: ${MIDLET_ICON}" >> ${JAD_PATH}; fi
	if test -n "${MIDLET_DESCRIPTION}"; then echo "MIDlet-Description: ${MIDLET_DESCRIPTION}" >> ${JAD_PATH}; fi
	if test -n "${MIDLET_INFO_URL}"; then echo "MIDlet-Info-URL: ${MIDLET_INFO_URL}" >> ${JAD_PATH}; fi
	# auの資料では必須項目なのだが…
	if test -n "${MIDLET_DATA_SIZE}"; then echo "MIDlet-Data-Size: ${MIDLET_DATA_SIZE}" >> ${JAD_PATH}; fi
}

case $1 in
	build)
		echo " * ビルドを開始しています..."
		manifest
#		compile
#		VERIFYCLASS_DIR=${TMPCLASS_DIR}
#		verify
		reduce
		makejar
		makejad
		echo " * ビルドが完了しました";;
	run)
		echo " * 実行しています..."
		${EMULATOR} -Xdescriptor:${JAD_PATH} -Xheapsize:1536K
		echo " * エミュレータが終了しました";;
	*)
		echo "MIDP tool"
		echo "Usage: $0 [ build | run ]";;
esac

exit

